struct Thermometer(i32);

const MOCK_TEMP: i32 = 42;

impl Thermometer {
    fn new() -> Self {
        // Set default value
        Thermometer(-273i32)
    }

    pub fn print_temp(&mut self) {
        println!("Temp {}", self.temp_as_string());
    }

    fn temp_as_string(&mut self) -> String {
        format!("{}", self.get_sensor_data())
    }

    fn get_sensor_data(&mut self) -> i32 {
        if MOCK_TEMP != self.0 {
            self.set_current_temp(MOCK_TEMP);
        }
        self.0
    }

    fn set_current_temp(&mut self, temp: i32) {
        self.0 = temp;
    }
}

enum PowerState {
    On,
    Off,
}

struct PowerSocket {
    state: PowerState,
    consumption: u32,
}

impl PowerSocket {
    fn new() -> Self {
        PowerSocket {
            state: PowerState::Off,
            consumption: 42,
        }
    }

    // Распечатывает текущий статус розетки на STDOUT
    fn print_status(&self) {
        let r = self.report();
        println!("{r}");
    }

    fn report(&self) -> String {
        let s: String = self.get_string_state();
        let c = self.get_consumption();

        format!(
            "---\nname: Power Socket 2000\ninfo:\n  status: {}\n  consumption: {}",
            s, c
        )
    }

    fn get_sensor_power_state(&self) -> u8 {
        1
    }

    fn is_power_on(&self) -> bool {
        if self.get_sensor_power_state() == 1 {
            return true;
        }
        false
    }

    // Получить текущий статус розетки
    fn get_state(&self) -> &PowerState {
        if self.is_power_on() {
            return &PowerState::On;
        }
        &PowerState::Off
    }

    fn get_string_state(&self) -> String {
        let s = match self.get_state() {
            PowerState::Off => String::from("off"),
            PowerState::On => String::from("on"),
        };
        s
    }

    // Получить данные с аппаратного счетчика
    fn get_sensor_data(&self) -> u32 {
        todo!()
    }

    // получить текущее потребление
    fn get_consumption(&self) -> u32 {
        self.consumption
    }

    // Установить текущее потребление
    fn set_consumption(&mut self, c: u32) {
        self.consumption = c;
    }

    // Включить розетку
    fn on(&mut self) {
        // power on code
        self.state = PowerState::On;
    }

    // Выключить розетку
    fn off(&mut self) {
        // power off code
        self.state = PowerState::Off;
    }

    // Должен пускаться тред
    fn watch_consumption(&mut self) {
        loop {
            let d = self.get_sensor_data();
            if d != self.consumption {
                self.set_consumption(d)
            }
        }
    }
}

pub enum Command {
    On,
    Off,
    GetStatus,
    GetConsumption,
    Report,
    Temp,
    Exit,
}

struct Server();

impl Server {
    fn recv(&self) -> Command {
        todo!()
    }

    pub fn send(&self, msg: String) {
        todo!("{msg}")
    }
}

struct IpcServer {
    ipc_server: Server,
    addr: String,
}

impl IpcServer {
    fn new(addr: String) -> Self {
        Self {
            ipc_server: Server(),
            addr,
        }
    }

    fn listen_for_commands(&mut self) {
        // Тут должен запускаться тред
        println!("Listen to commands on {}", self.addr);
        self._run();
    }

    fn recv(&self) -> Command {
        self.ipc_server.recv()
    }

    pub fn send(&self, msg: String) {
        self.ipc_server.send(msg)
    }

    fn _run(&mut self) {
        self.ipc_server = Server();
    }
}

fn main() {
    let mut t = Thermometer::new();
    t.print_temp();

    let mut s = PowerSocket::new();
    let mut ipc: IpcServer = IpcServer::new(String::from("unix:///run/pw.socket"));
    s.print_status();

    s.watch_consumption();
    ipc.listen_for_commands();
    loop {
        // Ждем пока не прилетит комманда в unix сокет.
        // Скорее всего в rust это выглядит немного по другому
        let cmd = ipc.recv();
        match cmd {
            Command::Exit => {
                s.off();
                std::process::exit(0x0);
            }
            Command::On => s.on(),
            Command::Off => s.off(),
            Command::GetStatus => ipc.send(s.get_string_state()),
            Command::GetConsumption => ipc.send(s.get_consumption().to_string()),
            Command::Report => ipc.send(s.report()),
            Command::Temp => t.print_temp(),
        };
    }
}
